<?php
// This file is part of Additional tools library for Moodle™.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

// phpcs:disable moodle.Files.BoilerplateComment.CommentEndedTooSoon

namespace tool_mulib\output;

/**
 * Action menu dropdown.
 *
 * @package     tool_mulib
 * @copyright   2024 Open LMS (https://www.openlms.net/)
 * @copyright   2025 Petr Skoda
 * @author      Petr Skoda
 * @license     https://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
class dropdown implements \core\output\renderable, \core\output\named_templatable {
    /** @var array $items links, dividers or custom html fragments */
    private $items = [];
    /** @var string */
    private $title;

    /**
     * Constructor.
     *
     * @param string $title
     */
    public function __construct(string $title) {
        $this->title = $title;
    }

    /**
     * Add standard link item to dropdown.
     *
     * @param string $label
     * @param \moodle_url $url
     */
    final public function add_item(string $label, \moodle_url $url): void {
        $this->items[] = ['label' => $label, 'url' => $url->out(false)];
    }

    /**
     * Add divider element.
     */
    final public function add_divider(): void {
        $this->items[] = ['divider' => true];
    }

    /**
     * Add link that opens dialog_form.
     *
     * @param \tool_mulib\output\dialog_form\link $link
     */
    final public function add_dialog_form(\tool_mulib\output\dialog_form\link $link): void {
        global $OUTPUT;
        $oldclass = $link->get_class();
        $link->set_class('dropdown-item');
        $this->items[] = ['customhtml' => $OUTPUT->render($link)];
        $link->set_class($oldclass);
    }

    /**
     * Are there any items?
     *
     * @return bool
     */
    final public function has_items(): bool {
        return !empty($this->items);
    }

    /**
     * Export data for template.
     *
     * @param \renderer_base $output
     * @return array
     */
    final public function export_for_template(\renderer_base $output): array {
        return [
            'title' => $this->title,
            'items' => $this->items,
        ];
    }

    /**
     * Get the name of the template to use for this templatable.
     *
     * @param \renderer_base $renderer The renderer requesting the template name
     * @return string
     */
    final public function get_template_name(\renderer_base $renderer): string {
        return 'tool_mulib/dropdown';
    }
}
